module ClockParser

export get_skew

function _parse_line(line::AbstractString)
    time, high, _ = split(line)
    parse(Float64, time), parse(Float64, high)
end

"""
    get_skew(file::T) where {T <: AbstractString} -> Union{Float64, Missing}

Get skew from clock result file `file`.
It returns `missing` for invalid files.
To see the detailed reason, increase log level to `DEBUG`.
Invalid cases are:

- first line has >0 counts => "Unexpected first line"
- no measurement has >500 counts => "Clock skew out of range"
"""
function get_skew(file::T) where {T <: AbstractString}
    lines = Iterators.Stateful(eachline(file))

    first_line = popfirst!(lines)
    # criteria: https://gitlab.cern.ch/dhashimo/PS_Board_QAQC/-/blob/master/Software_Test/project_auto_reset/QAQC_test_func.cxx?ref_type=heads#L208
    let
        time, high = _parse_line(first_line)
        if high > 0
            @debug "Unexpected first line" file
            return missing
        elseif high >= 500
            return time
        end
    end

    time_and_counts = Iterators.map(_parse_line, lines)
    for (time, high) in time_and_counts
        if high >= 500
            return time
        end
    end
    @debug "Clock skew out of range"
    return missing
end

"""
    search_oscillation(file::T) where {T <: AbstractString}

Search oscillation (two or more rise up) for clock measurement file.
"""
function count_riseup(file::T) where {T <: AbstractString}
    lines = Iterators.Stateful(eachline(file))

    rising_count = 0
    first_line = popfirst!(lines)
    is_high = let
        time, high = _parse_line(first_line)
        high >= 500
    end

    time_and_counts = Iterators.map(_parse_line, lines)
    for (time, high) in time_and_counts
        if !is_high && high >= 500
            is_high = true
            rising_count += 1
        elseif is_high && high < 500
            is_high = false
        end
    end

    return rising_count

    # lines = eachline(file)
    # time_and_counts = Iterators.map(_parse_line, lines)
    # is_high = Iterators.map(time_and_counts) do (time, counts)
    #     counts >= 500
    # end
    # edges = Iterators.map(accumulate((p, n) -> (n, !p[1] & n), is_high, init = (false, false))) do x
    #     _prev, edge = x
    #     edge
    # end
    #
    # return sum(edges)
end

end # module ClockParser
